@@ -2218,7 +2218,7 @@ void QuattroSpreadsheet::sendCellContent(QuattroSpreadsheetInternal::Cell const
 				((uint32_t)libwps_tools_win::Font::unicode((unsigned char)text[c],fontType), finalString);
 			else
 				libwps::appendUnicode
-				((uint32_t)libwps_tools_win::Font::LICSunicode((unsigned char)text[c],fontType), finalString);
+				((uint32_t)libwps_tools_win::Font::LICSunicode((unsigned char)text[c]), finalString);
 		}
 		text=finalString.cstr();
 	}
@@ -2250,7 +2250,7 @@ void QuattroSpreadsheet::sendCellContent(QuattroSpreadsheetInternal::Cell const
 				if (!hasLICS)
 					m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::unicode(c,fontType));
 				else
-					m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::LICSunicode(c,fontType));
+					m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::LICSunicode(c));
 				prevEOL=false;
 			}
 		}
