@@ -39,6 +39,7 @@
 #include "WPSHeader.h"
 #include "WPSPageSpan.h"
 
+#include "WKS4Format.h"
 #include "WKS4Spreadsheet.h"
 
 #include "WKS4.h"
@@ -292,7 +293,10 @@ void WKS4Parser::parse(librevenge::RVNGSpreadsheetInterface *documentInterface)
 		ascii().open("MN0");
 
 		if (checkHeader(0L) && readZones())
+		{
+			parseFormatStream();
 			m_listener=createListener(documentInterface);
+		}
 		if (m_listener)
 		{
 			m_spreadsheetParser->setListener(m_listener);
@@ -475,6 +479,22 @@ bool WKS4Parser::checkHeader(WPSHeader *header, bool strict)
 	return true;
 }
 
+bool WKS4Parser::parseFormatStream()
+{
+	RVNGInputStreamPtr file=getFileInput();
+	if (!file || !file->isStructured() || !m_state->m_isSpreadsheet)
+		return false;
+
+	RVNGInputStreamPtr formatInput(file->getSubStreamByName("FMT"));
+	if (!formatInput)
+	{
+		WPS_DEBUG_MSG(("WKS4Parser::parseFormatStream: can not find the format stream\n"));
+		return false;
+	}
+	WKS4Format formatManager(*this, formatInput);
+	return formatManager.parse();
+}
+
 bool WKS4Parser::readZones()
 {
 	RVNGInputStreamPtr input = getInput();
@@ -637,6 +657,7 @@ bool WKS4Parser::readZone()
 		case 0xe: // floating cell
 		case 0xf: // label cell
 		case 0x10: // formula cell
+		case 0x36: // continue label
 			ok = m_spreadsheetParser->readCell();
 			isParsed = true;
 			break;
@@ -757,19 +778,6 @@ bool WKS4Parser::readZone()
 				m_state->m_version=1;
 			isParsed = needWriteInAscii = true;
 			break;
-		case 0x36: // find one time in a spreadsheet (not standard)
-		{
-			if (sz!=0x1e) break;
-			input->seek(pos+4, librevenge::RVNG_SEEK_SET);
-			for (int i=0; i<3; ++i)   // always 0?
-			{
-				val=(int) libwps::read16(input);
-				if (val) f << "f" << i << "=" << val << ",";
-			}
-			// after find some junk, so..
-			isParsed = needWriteInAscii = true;
-			break;
-		}
 		case 0x41: // graph record name
 			readChartName();
 			isParsed = true;
@@ -1285,7 +1293,7 @@ bool WKS4Parser::readFont()
 	uint32_t attributes = 0;
 	if (flags & 1) attributes |= WPS_BOLD_BIT;
 	if (flags & 2) attributes |= WPS_ITALICS_BIT;
-	if (flags & 4) attributes |= WPS_BOLD_BIT;
+	if (flags & 4) attributes |= WPS_UNDERLINE_BIT;
 	if (flags & 8) attributes |= WPS_STRIKEOUT_BIT;
 
 	font.m_attributes=attributes;
@@ -1362,7 +1370,7 @@ void WKS4Parser::sendHeaderFooter(bool header)
 		else if (!hasLICS)
 			m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::unicode(c,fontType));
 		else
-			m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::LICSunicode(c,fontType));
+			m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::LICSunicode(c));
 	}
 
 }
