@@ -174,7 +174,7 @@ class Cell : public WPSCell
 {
 public:
 	/// constructor
-	Cell() : m_styleId(-1), m_hAlign(WPSCellFormat::HALIGN_DEFAULT), m_content() { }
+	Cell() : m_styleId(-1), m_hAlign(WPSCellFormat::HALIGN_DEFAULT), m_content(), m_extraTextEntryList() { }
 
 	//! operator<<
 	friend std::ostream &operator<<(std::ostream &o, Cell const &cell);
@@ -199,6 +199,10 @@ public:
 	WPSCellFormat::HorizontalAlignment m_hAlign;
 	//! the content
 	WKSContentListener::CellContent m_content;
+	/** As very long text is splitted in zone 0xf and then in zone 0x36,
+		the list of zone36 text entries...
+	 */
+	std::vector<WPSEntry> m_extraTextEntryList;
 };
 
 //! operator<<
@@ -584,7 +588,7 @@ bool WKS4Spreadsheet::readColumnSize()
 			f << "#col[inc],";
 		}
 		// checkme: unit in character(?) -> TWIP
-		m_state->getActualSheet().setColumnWidth(col, width*105);
+		m_state->getActualSheet().setColumnWidth(col, width*160);
 	}
 	ascii().addPos(pos);
 	ascii().addNote(f.str().c_str());
@@ -792,8 +796,10 @@ bool WKS4Spreadsheet::readMsWorksStyle()
 		case 4: // left rellenar
 			style.setHAlignement(WPSCell::HALIGN_LEFT);
 			break;
+		case 5: // center arround cell
+			f << ",center[between cell]";
+		// fail through expected
 		case 2:
-		case 5: // center in selection
 			style.setHAlignement(WPSCell::HALIGN_CENTER);
 			break;
 		case 3:
@@ -1353,7 +1359,7 @@ bool WKS4Spreadsheet::readCell()
 
 	long pos = m_input->tell();
 	long type = libwps::read16(m_input);
-	if ((type != 0x545b) && (type < 0xc || type > 0x10))
+	if ((type != 0x545b) && (type!=0x36) && (type < 0xc || type > 0x10))
 	{
 		WPS_DEBUG_MSG(("WKS4Spreadsheet::readCell: not a cell property\n"));
 		return false;
@@ -1446,7 +1452,14 @@ bool WKS4Spreadsheet::readCell()
 		break;
 	}
 	case 15:
+	case 0x36: // continue text zone
 	{
+		if (type==0x36 && cell.m_content.m_contentType!=WKSContentListener::CellContent::C_TEXT)
+		{
+			WPS_DEBUG_MSG(("WKS4Spreadsheet::readCell: oops, find zone 0x36 but not zone 15\n"));
+			f << "###";
+			break;
+		}
 		cell.m_content.m_contentType=WKSContentListener::CellContent::C_TEXT;
 		long begText=m_input->tell(), endText=begText+dataSz;
 		std::string s("");
@@ -1474,8 +1487,18 @@ bool WKS4Spreadsheet::readCell()
 				--begText;
 			++begText;
 		}
-		cell.m_content.m_textEntry.setBegin(begText);
-		cell.m_content.m_textEntry.setEnd(endText);
+		if (type==15)
+		{
+			cell.m_content.m_textEntry.setBegin(begText);
+			cell.m_content.m_textEntry.setEnd(endText);
+		}
+		else
+		{
+			WPSEntry newEntry;
+			newEntry.setBegin(begText);
+			newEntry.setEnd(endText);
+			cell.m_extraTextEntryList.push_back(newEntry);
+		}
 		break;
 	}
 	case 16:
@@ -1809,7 +1832,7 @@ static Functions const s_listFunctions[] =
 
 	{ "Time", 3},{ "Hour", 1},{ "Minute", 1},{ "Second", 1},
 	{ "IsNumber", 1},{ "IsText", 1},{ "Len", 1},{ "Value", 1},
-	{ "Text", 2}/* or fixed*/, { "Mid", 3}, { "Char", 1},{ "Ascii", 1},
+	{ "Fixed", 2}, { "Mid", 3}, { "Char", 1},{ "Ascii", 1},
 	{ "Find", 3},{ "DateValue", 1} /*checkme*/,{ "TimeValue", 1} /*checkme*/,{ "CellPointer", 1} /*checkme*/,
 
 	{ "Sum", -1},{ "Average", -1},{ "COUNT", -1},{ "Min", -1},
@@ -2299,7 +2322,7 @@ void WKS4Spreadsheet::sendSpreadsheet(int sId)
 		sheet.reset(new WKS4SpreadsheetInternal::Spreadsheet);
 	}
 
-	m_listener->openSheet(sheet->convertInPoint(sheet->m_widthCols,76), librevenge::RVNG_POINT,
+	m_listener->openSheet(sheet->convertInPoint(sheet->m_widthCols,72), librevenge::RVNG_POINT,
 	                      std::vector<int>(), m_state->getSheetName(sId));
 	sheet->compressRowHeights();
 	std::map<Vec2i, WKS4SpreadsheetInternal::Cell>::const_iterator it = sheet->m_positionToCellMap.begin();
@@ -2369,17 +2392,22 @@ void WKS4Spreadsheet::sendCellContent(WKS4SpreadsheetInternal::Cell const &cell)
 				((uint32_t)libwps_tools_win::Font::unicode((unsigned char)text[c],fontType), finalString);
 			else
 				libwps::appendUnicode
-				((uint32_t)libwps_tools_win::Font::LICSunicode((unsigned char)text[c],fontType), finalString);
+				((uint32_t)libwps_tools_win::Font::LICSunicode((unsigned char)text[c]), finalString);
 		}
 		text=finalString.cstr();
 	}
 	m_listener->openSheetCell(finalCell, content);
 
-	if (cell.m_content.m_textEntry.valid())
+	size_t numTextEntry=cell.m_content.m_textEntry.valid() ? 1 : 0;
+	if (!cell.m_extraTextEntryList.empty())
+		numTextEntry=1+cell.m_extraTextEntryList.size();
+	for (size_t t=0; t<numTextEntry; ++t)
 	{
-		m_input->seek(cell.m_content.m_textEntry.begin(), librevenge::RVNG_SEEK_SET);
+		WPSEntry entry=t==0 ? cell.m_content.m_textEntry : cell.m_extraTextEntryList[t-1];
+		if (!entry.valid()) continue;
+		m_input->seek(entry.begin(), librevenge::RVNG_SEEK_SET);
 		bool prevEOL=false;
-		while (!m_input->isEnd() && m_input->tell()<cell.m_content.m_textEntry.end())
+		while (!m_input->isEnd() && m_input->tell()<entry.end())
 		{
 			unsigned char c=(unsigned char) libwps::readU8(m_input);
 			if (c==0xd)
@@ -2400,7 +2428,7 @@ void WKS4Spreadsheet::sendCellContent(WKS4SpreadsheetInternal::Cell const &cell)
 				if (!hasLICS)
 					m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::unicode(c,fontType));
 				else
-					m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::LICSunicode(c,fontType));
+					m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::LICSunicode(c));
 				prevEOL=false;
 			}
 		}
