@@ -47,7 +47,7 @@
 struct WKSDocumentParsingState
 {
 	//! constructor
-	WKSDocumentParsingState(std::vector<WPSPageSpan> const &pageList);
+	explicit WKSDocumentParsingState(std::vector<WPSPageSpan> const &pageList);
 	//! destructor
 	~WKSDocumentParsingState();
 
@@ -102,6 +102,8 @@ struct WKSContentParsingState
 
 	WPSFont m_font;
 	WPSParagraph m_paragraph;
+	//! a flag to know if openGroup was called
+	bool m_isGroupOpened;
 
 	bool m_isParagraphColumnBreak;
 	bool m_isParagraphPageBreak;
@@ -132,7 +134,7 @@ WKSContentParsingState::WKSContentParsingState() :
 	m_textBuffer(""), m_numDeferredTabs(0),
 
 	m_font(), m_paragraph(),
-
+	m_isGroupOpened(false),
 	m_isParagraphColumnBreak(false), m_isParagraphPageBreak(false),
 
 	m_isSpanOpened(false), m_isParagraphOpened(false),
@@ -183,7 +185,7 @@ void WKSContentListener::insertUnicode(uint32_t val)
 	if (val == 0xfffd) return;
 	_flushDeferredTabs();
 	if (!m_ps->m_isSpanOpened) _openSpan();
-	appendUnicode(val, m_ps->m_textBuffer);
+	libwps::appendUnicode(val, m_ps->m_textBuffer);
 }
 
 void WKSContentListener::insertUnicodeString(librevenge::RVNGString const &str)
@@ -298,62 +300,22 @@ void WKSContentListener::setParagraph(const WPSParagraph &para)
 ///////////////////
 // field :
 ///////////////////
-#include <time.h>
-void WKSContentListener::insertField(WKSContentListener::FieldType type)
+void WKSContentListener::insertField(WPSField const &field)
 {
-	switch (type)
-	{
-	case None:
-		break;
-	case PageNumber:
+	librevenge::RVNGPropertyList propList;
+	if (field.addTo(propList))
 	{
 		_flushText();
 		_openSpan();
-		librevenge::RVNGPropertyList propList;
-		propList.insert("style:num-format", libwps::numberingTypeToString(libwps::ARABIC).c_str());
-		propList.insert("librevenge:field-type", "text:page-number");
 		m_documentInterface->insertField(propList);
-		break;
-	}
-	case Database:
-	{
-		librevenge::RVNGString tmp("#DATAFIELD#");
-		insertUnicodeString(tmp);
-		break;
-	}
-	case Title:
-	{
-		librevenge::RVNGString tmp("#TITLE#");
-		insertUnicodeString(tmp);
-		break;
-	}
-	case Date:
-		insertDateTimeField("%m/%d/%y");
-		break;
-	case Time:
-		insertDateTimeField("%I:%M:%S %p");
-		break;
-	case Link:
-	default:
-		WPS_DEBUG_MSG(("WKSContentListener::insertField: must not be called with type=%d\n", int(type)));
-		break;
-	}
-}
-
-void WKSContentListener::insertDateTimeField(char const *format)
-{
-	if (!format)
-	{
-		WPS_DEBUG_MSG(("WKSContentListener::insertDateTimeField: oops, can not find the format\n"));
 		return;
 	}
-	time_t now = time(0L);
-	struct tm timeinfo;
-	if (localtime_r(&now, &timeinfo))
+	librevenge::RVNGString text=field.getString();
+	if (!text.empty())
+		WKSContentListener::insertUnicodeString(text);
+	else
 	{
-		char buf[256];
-		strftime(buf, 256, format, &timeinfo);
-		insertUnicodeString(librevenge::RVNGString(buf));
+		WPS_DEBUG_MSG(("WKSContentListener::insertField: must not be called with type=%d\n", int(field.m_type)));
 	}
 }
 
@@ -368,6 +330,16 @@ void WKSContentListener::setDocumentLanguage(int lcid)
 	m_ds->m_metaData.insert("librevenge:language", lang.c_str());
 }
 
+void WKSContentListener::setMetaData(const librevenge::RVNGPropertyList &list)
+{
+	librevenge::RVNGPropertyList::Iter i(list);
+
+	for (i.rewind(); i.next();)
+	{
+		m_ds->m_metaData.insert(i.key(), i()->getStr());
+	}
+}
+
 void WKSContentListener::startDocument()
 {
 	if (m_ds->m_isDocumentStarted)
@@ -556,13 +528,18 @@ void WKSContentListener::insertComment(WPSSubDocumentPtr &subDocument)
 		return;
 	}
 
-	if (!m_ps->m_isParagraphOpened)
-		_openParagraph();
-	else
+	if (!m_ps->m_isSheetCellOpened)
 	{
-		_flushText();
-		_closeSpan();
+		if (!m_ps->m_isParagraphOpened)
+			_openParagraph();
+		else
+		{
+			_flushText();
+			_closeSpan();
+		}
 	}
+	else if (m_ps->m_isParagraphOpened)
+		_closeParagraph();
 
 	librevenge::RVNGPropertyList propList;
 	m_documentInterface->openComment(propList);
@@ -601,6 +578,18 @@ void WKSContentListener::insertPicture
 	_closeFrame();
 }
 
+void WKSContentListener::insertObject
+(WPSPosition const &pos, const WPSEmbeddedObject &obj, WPSGraphicStyle const &style)
+{
+	if (!_openFrame(pos, style)) return;
+
+	librevenge::RVNGPropertyList propList;
+	if (obj.addTo(propList))
+		m_documentInterface->insertBinaryObject(propList);
+
+	_closeFrame();
+}
+
 void WKSContentListener::insertPicture
 (WPSPosition const &pos, const WPSGraphicShape &shape, WPSGraphicStyle const &style)
 {
@@ -646,6 +635,42 @@ void WKSContentListener::insertPicture
 ///////////////////
 // frame
 ///////////////////
+bool WKSContentListener::openGroup(WPSPosition const &pos)
+{
+	if (!m_ds->m_isDocumentStarted)
+	{
+		WPS_DEBUG_MSG(("WKSContentListener::openGroup: the document is not started\n"));
+		return false;
+	}
+	if (m_ps->m_isSheetRowOpened)
+	{
+		WPS_DEBUG_MSG(("WKSContentListener::openGroup: can not open a group\n"));
+		return false;
+	}
+	librevenge::RVNGPropertyList propList;
+	_handleFrameParameters(propList, pos);
+
+	_pushParsingState();
+	_startSubDocument();
+	m_ps->m_isGroupOpened = true;
+
+	m_documentInterface->openGroup(propList);
+
+	return true;
+}
+
+void WKSContentListener::closeGroup()
+{
+	if (!m_ps->m_isGroupOpened)
+	{
+		WPS_DEBUG_MSG(("WKSContentListener::closeGroup: called but no group is already opened\n"));
+		return;
+	}
+	_endSubDocument();
+	_popParsingState();
+	m_documentInterface->closeGroup();
+}
+
 bool WKSContentListener::_openFrame(WPSPosition const &pos, WPSGraphicStyle const &style)
 {
 	if (m_ps->m_isFrameOpened)
@@ -657,8 +682,10 @@ bool WKSContentListener::_openFrame(WPSPosition const &pos, WPSGraphicStyle cons
 	switch (pos.m_anchorTo)
 	{
 	case WPSPosition::Page:
+	case WPSPosition::PageContent:
 		break;
 	case WPSPosition::Paragraph:
+	case WPSPosition::ParagraphContent:
 		if (m_ps->m_isParagraphOpened)
 			_flushText();
 		else
@@ -724,7 +751,7 @@ void WKSContentListener::_handleFrameParameters
 	else
 		propList.insert("style:wrap", "none");
 
-	if (pos.m_anchorTo != WPSPosition::Page)
+	if (pos.m_anchorTo != WPSPosition::Page && pos.m_anchorTo != WPSPosition::PageContent)
 	{
 		WPS_DEBUG_MSG(("WKSContentListener::openFrame: only implemented for page anchor\n"));
 		return;
@@ -738,8 +765,9 @@ void WKSContentListener::_handleFrameParameters
 	w *= inchFactor;
 	h *= inchFactor;
 
-	propList.insert("style:vertical-rel", "page");
-	propList.insert("style:horizontal-rel", "page");
+	librevenge::RVNGString relPos(pos.m_anchorTo == WPSPosition::Page ? "page" : "page-content");
+	propList.insert("style:vertical-rel", relPos);
+	propList.insert("style:horizontal-rel", relPos);
 
 	double newPosition;
 	switch (pos.m_yPos)
@@ -891,7 +919,8 @@ void WKSContentListener::_endSubDocument()
 ///////////////////
 // sheet
 ///////////////////
-void WKSContentListener::openSheet(std::vector<float> const &colWidth, librevenge::RVNGUnit unit, librevenge::RVNGString const &name)
+void WKSContentListener::openSheet(std::vector<float> const &colWidth, librevenge::RVNGUnit unit,
+                                   std::vector<int> const &repeatColWidthNumber, librevenge::RVNGString const &name)
 {
 	if (m_ps->m_isSheetOpened)
 	{
@@ -913,10 +942,20 @@ void WKSContentListener::openSheet(std::vector<float> const &colWidth, libreveng
 	librevenge::RVNGPropertyListVector columns;
 
 	size_t nCols = colWidth.size();
+	bool useRepeated=repeatColWidthNumber.size()==nCols;
+	if (!useRepeated&&!repeatColWidthNumber.empty())
+	{
+		WPS_DEBUG_MSG(("WKSContentListener::openSheet: repeatColWidthNumber seems bad\n"));
+	}
 	for (size_t c = 0; c < nCols; c++)
 	{
 		librevenge::RVNGPropertyList column;
-		column.insert("style:column-width", colWidth[c], unit);
+		if (colWidth[c]>=0)
+			column.insert("style:column-width", colWidth[c], unit);
+		else
+			column.insert("style:use-optimal-column-width", true);
+		if (useRepeated && repeatColWidthNumber[c]>1)
+			column.insert("table:number-columns-repeated", repeatColWidthNumber[c]);
 		columns.append(column);
 	}
 	propList.insert("librevenge:columns", columns);
@@ -941,7 +980,7 @@ void WKSContentListener::closeSheet()
 	_popParsingState();
 }
 
-void WKSContentListener::openSheetRow(float h, librevenge::RVNGUnit unit, bool headerRow)
+void WKSContentListener::openSheetRow(float h, librevenge::RVNGUnit unit, bool headerRow, int numRepeated)
 {
 	if (m_ps->m_isSheetRowOpened)
 	{
@@ -955,6 +994,8 @@ void WKSContentListener::openSheetRow(float h, librevenge::RVNGUnit unit, bool h
 	}
 	librevenge::RVNGPropertyList propList;
 	propList.insert("librevenge:is-header-row", headerRow);
+	if (numRepeated>1)
+		propList.insert("table:number-rows-repeated", numRepeated);
 
 	if (h > 0)
 		propList.insert("style:row-height", h, unit);
@@ -975,7 +1016,7 @@ void WKSContentListener::closeSheetRow()
 	m_documentInterface->closeSheetRow();
 }
 
-void WKSContentListener::openSheetCell(WPSCell const &cell, WKSContentListener::CellContent const &content, librevenge::RVNGPropertyList const &extras)
+void WKSContentListener::openSheetCell(WPSCell const &cell, WKSContentListener::CellContent const &content, int numRepeated)
 {
 	if (!m_ps->m_isSheetRowOpened)
 	{
@@ -988,8 +1029,10 @@ void WKSContentListener::openSheetCell(WPSCell const &cell, WKSContentListener::
 		closeSheetCell();
 	}
 
-	librevenge::RVNGPropertyList propList(extras);
+	librevenge::RVNGPropertyList propList;
 	cell.addTo(propList);
+	if (numRepeated>1)
+		propList.insert("table:number-columns-repeated", numRepeated);
 	cell.getFont().addTo(propList);
 	if (!cell.hasBasicFormat())
 	{
@@ -1297,14 +1340,57 @@ std::ostream &operator<<(std::ostream &o, WKSContentListener::FormulaInstruction
 // ---------- WKSContentListener::CellContent ------------------
 bool WKSContentListener::CellContent::double2Date(double val, int &Y, int &M, int &D)
 {
-	// number of day since 1/1/1970
-	time_t date= time_t((val-24107-1462+0.4)*24.*3600);
-	struct tm dateTm;
-	if (!gmtime_r(&date,&dateTm)) return false;
+	/* first convert the date in long. Checkme: unsure why -2 is needed...*/
+	long numDaysSinceOrigin=long(val-2+0.4);
+	// checkme: do we need to check before for isNan(val) ?
+	if (numDaysSinceOrigin<-10000*365 || numDaysSinceOrigin>10000*365)
+	{
+		/* normally, we can expect documents to contain date between 1904
+		   and 2004. So even if such a date can make sense, storing it as
+		   a number of days is clearly abnormal */
+		WPS_DEBUG_MSG(("WKSContentListener::CellContent::double2Date: using a double to represent the date %ld seems odd\n", numDaysSinceOrigin));
+		Y=1904;
+		M=D=1;
+		return false;
+	}
+	// find the century
+	int century=19;
+	while (numDaysSinceOrigin>=36500+24)
+	{
+		long numDaysInCentury=36500+24+((century%4)?0:1);
+		if (numDaysSinceOrigin<numDaysInCentury) break;
+		numDaysSinceOrigin-=numDaysInCentury;
+		++century;
+	}
+	while (numDaysSinceOrigin<0)
+	{
+		--century;
+		numDaysSinceOrigin+=36500+24+((century%4)?0:1);
+	}
+	// now compute the year
+	Y=int(numDaysSinceOrigin/365);
+	long numDaysToEndY1=Y*365+(Y>0 ? (Y-1)/4+((century%4)?0:1) : 0);
+	if (numDaysToEndY1>numDaysSinceOrigin)
+	{
+		--Y;
+		numDaysToEndY1=Y*365+(Y>0 ? (Y-1)/4+((century%4)?0:1) : 0);
+	}
+	// finish to compute the date
+	int numDaysFrom1Jan=int(numDaysSinceOrigin-numDaysToEndY1);
+	Y+=century*100;
+	bool isLeap=(Y%4)==0 && ((Y%400)==0 || (Y%100)!=0);
 
-	Y = dateTm.tm_year+1900;
-	M=dateTm.tm_mon+1;
-	D=dateTm.tm_mday;
+	for (M=0; M<12; ++M)
+	{
+		static const int days[2][12] =
+		{
+			{ 0,31,59,90,120,151,181,212,243,273,304,334},
+			{ 0,31,60,91,121,152,182,213,244,274,305,335}
+		};
+		if (M<11 && days[isLeap ? 1 : 0][M+1]<=numDaysFrom1Jan) continue;
+		D=(numDaysFrom1Jan-days[isLeap ? 1 : 0][M++])+1;
+		break;
+	}
 	return true;
 }
 
