@@ -53,7 +53,8 @@ class WPS4Parser : public WPSParser
 
 public:
 	//! constructor
-	WPS4Parser(RVNGInputStreamPtr &input, WPSHeaderPtr &header);
+	WPS4Parser(RVNGInputStreamPtr &input, WPSHeaderPtr &header,
+	           libwps_tools_win::Font::Type encoding=libwps_tools_win::Font::UNKNOWN);
 	//! destructor
 	~WPS4Parser();
 	//! called by WPSDocument to parse the file
@@ -62,7 +63,7 @@ public:
 	bool checkHeader(WPSHeader *header, bool strict=false);
 protected:
 	//! color
-	bool getColor(int id, uint32_t &color) const;
+	bool getColor(int id, WPSColor &color) const;
 
 	//! sets the file size ( filled by WPS4Text )
 	void setSizeFile(long sz);
@@ -89,8 +90,12 @@ protected:
 	float pageWidth() const;
 	//! returns the number of columns
 	int numColumns() const;
-	//! returns the document codepage ( if known )
-	libwps_tools_win::Font::Type getDocumentFontType() const;
+	/** returns the default font type, ie. the encoding given by the constructor if given
+		or the encoding deduiced from the version.
+	 */
+	libwps_tools_win::Font::Type getDefaultFontType() const;
+	//! returns the document codepage ( given from the file OEM field )
+	libwps_tools_win::Font::Type getOEMFontType() const;
 
 	/** creates a document for a comment entry and then send the data
 	 *
@@ -112,8 +117,8 @@ protected:
 
 	/** sends an object with given id ( via its WPS4Graph )
 	 *
-	 * The object is sent as a character with given size \a size */
-	void sendObject(Vec2f const &size, int id);
+	 * The object is sent as a character with given size \a position */
+	void sendObject(WPSPosition const &position, int id);
 
 	//
 	// low level
