@@ -19,6 +19,7 @@
  * applicable instead of those above.
  */
 
+#include <algorithm>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -132,7 +133,7 @@ struct Frame
 	//! constructor
 	Frame() : m_parsed(false), m_type(UNKNOWN), m_pos(),
 		m_idStrs(-1), m_idObject(-1), m_idTable(-1), m_idOle(-1), m_columns(1), m_idBorder(),
-		m_backgroundColor(0xFFFFFF), m_error("")
+		m_backgroundColor(WPSColor::white()), m_error("")
 	{
 		m_pos.setRelativePosition(WPSPosition::Page);
 		m_pos.setPage(1);
@@ -147,7 +148,7 @@ struct Frame
 	 * - Table : a table
 	 * - Object : a picture, ...
 	 * - Text : a text zone */
-	enum { UNKNOWN = 0, Header, Footer, Table, Object, Text };
+	enum { UNKNOWN = 0, DbField, Header, Footer, Table, Object, Text };
 
 	//! a flag to know if the frame is already sent to the listener
 	mutable bool m_parsed;
@@ -164,7 +165,7 @@ struct Frame
 	//! the border: an entry to some complex border (if sets)
 	WPSEntry m_idBorder;
 	//! the border's color
-	uint32_t m_backgroundColor;
+	WPSColor m_backgroundColor;
 	//! a string used to store the parsing errors
 	std::string m_error;
 };
@@ -174,6 +175,9 @@ std::ostream &operator<<(std::ostream &o, Frame const &ft)
 {
 	switch (ft.m_type)
 	{
+	case Frame::DbField:
+		o<< "db[field]";
+		break;
 	case Frame::Header:
 		o<< "header";
 		break;
@@ -215,8 +219,8 @@ std::ostream &operator<<(std::ostream &o, Frame const &ft)
 	if (ft.m_columns != 1) o << ft.m_columns << "columns,";
 	if (ft.m_idBorder.valid())
 		o << "border='" << ft.m_idBorder.name() << "':" << ft.m_idBorder.id() << ",";
-	if (ft.m_backgroundColor != 0xFFFFFF)
-		o << "backColor=" << std::hex << ft.m_backgroundColor << std::dec << ",";
+	if (!ft.m_backgroundColor.isWhite())
+		o << "backColor=" << ft.m_backgroundColor << ",";
 
 	if (!ft.m_error.empty()) o << "errors=(" << ft.m_error << ")";
 	return o;
@@ -275,7 +279,7 @@ void State::initTypeMaps()
 	{
 		0, 0x1a, 1, 0x12, 2, 0x12, 3, 0x2, 4, 0x22, 5, 0x22, 6, 0x22, 7, 0x22,
 		8, 0x22, 9, 0x22, 0xa, 0x22,
-		0x10, 0x2a, 0x11, 0x82, 0x13, 0x12, 0x14, 0x12, 0x17, 0x2,
+		0x10, 0x2a, 0x11, 0x82, 0x13, 0x12, 0x14, 0x12, 0x16, 0x2a, 0x17, 0x2,
 		0x18, 0x22, 0x19, 0x22, 0x1a, 0x12, 0x1b, 0x22, 0x1d, 0x22, 0x1e, 0x22, 0x1f, 0x22,
 		0x20, 0x22, 0x26, 0x22,
 		0x2a, 0x22, 0x2c, 0x1a, 0x2d, 0x1a, 0x2e, 0x22, 0x2f, 0x2,
@@ -285,12 +289,37 @@ void State::initTypeMaps()
 		m_frameTypes[frameTypes[i]] = frameTypes[i+1];
 
 }
+
+class TableRecursionGuard
+{
+	// disable copying
+	TableRecursionGuard(const TableRecursionGuard &);
+	TableRecursionGuard &operator=(const TableRecursionGuard &);
+
+public:
+	TableRecursionGuard(std::deque<int> &stack, const int id)
+		: m_stack(stack)
+	{
+		m_stack.push_front(id);
+	}
+
+	~TableRecursionGuard()
+	{
+		m_stack.pop_front();
+	}
+
+private:
+	std::deque<int> &m_stack;
+};
+
+
 }
 
 // constructor, destructor
 WPS8Parser::WPS8Parser(RVNGInputStreamPtr &input, WPSHeaderPtr &header) :
 	WPSParser(input, header),
-	m_listener(), m_graphParser(), m_tableParser(), m_textParser(), m_state()
+	m_listener(), m_graphParser(), m_tableParser(), m_textParser(), m_state(),
+	m_sendingTables()
 {
 	if (version()<5) setVersion(5);
 	m_state.reset(new WPS8ParserInternal::State);
@@ -450,6 +479,12 @@ bool WPS8Parser::sendObject(Vec2f const &size, int objectId, bool ole)
 
 bool WPS8Parser::sendTable(Vec2f const &size, int objectId)
 {
+	if (find(m_sendingTables.begin(), m_sendingTables.end(), objectId) != m_sendingTables.end())
+	{
+		WPS_DEBUG_MSG(("WPS8Parser::sendTable table %d is contained in itself\n", objectId));
+		return false;
+	}
+
 	std::map<int, int>::iterator pos = m_state->m_object2FrameMap.find(objectId);
 	if (pos == m_state->m_object2FrameMap.end())
 	{
@@ -475,6 +510,7 @@ bool WPS8Parser::sendTable(Vec2f const &size, int objectId)
 		return true;
 	}
 	frame.m_parsed = true;
+	WPS8ParserInternal::TableRecursionGuard guard(m_sendingTables, objectId);
 	return m_tableParser->sendTable(size, frame.m_idTable, frame.m_idStrs);
 }
 
@@ -627,6 +663,16 @@ bool WPS8Parser::createStructures()
 
 		readFRAM(entry);
 	}
+	// FRCD
+	pos = getNameEntryMap().lower_bound("FRCD");
+	while (getNameEntryMap().end() != pos)
+	{
+		WPSEntry const &entry = pos++->second;
+		if (!entry.hasName("FRCD")) break;
+		if (!entry.hasType("FRCD")) continue;
+		readFRCD(entry);
+	}
+
 	// creates the correspondance between the eobj and the frame
 	size_t numFrames = m_state->m_frameList.size();
 	for (size_t i = 0; i < numFrames; i++)
@@ -637,7 +683,6 @@ bool WPS8Parser::createStructures()
 		m_state->m_table2FrameMap[frame.m_idTable] = (int) i;
 	}
 
-
 	m_graphParser->computePositions();
 
 	return true;
@@ -712,6 +757,7 @@ void WPS8Parser::sendPageFrames()
 		WPSPosition pos(frame.m_pos);
 		if (pos.origin().y() < 0 || pos.origin().x() < 0)
 		{
+			if (frame.m_type == Frame::DbField) continue;
 			// FIXME: the
 			WPS_DEBUG_MSG(("WPS8Parser::insertPageFrames: origin is too small\n"));
 			Vec2f orig = pos.origin();
@@ -727,12 +773,8 @@ void WPS8Parser::sendPageFrames()
 		else if (frame.m_type == Frame::Text)
 		{
 			librevenge::RVNGPropertyList frameExtras;
-			if (frame.m_backgroundColor != 0xFFFFFF)
-			{
-				char color[20];
-				sprintf(color,"#%06x",frame.m_backgroundColor);
-				frameExtras.insert("fo:background-color", color);
-			}
+			if (!frame.m_backgroundColor.isWhite())
+				frameExtras.insert("fo:background-color", frame.m_backgroundColor.str().c_str());
 			sendTextBox(pos, frame.m_idStrs, frameExtras);
 		}
 		else if (frame.m_type != Frame::Header && frame.m_type != Frame::Footer)
@@ -788,7 +830,7 @@ bool WPS8Parser::parseHeaderIndexEntry()
 
 		if (c != 0 && c != 0x20 && (41 > c || c > 90))
 		{
-			WPS_DEBUG_MSG(("WPS8Parser::parseHeaderIndexEntry: error: bad character=%u (0x%02x) in name in header index\n", c, c));
+			WPS_DEBUG_MSG(("WPS8Parser::parseHeaderIndexEntry: error: bad character=%u (0x%02x) in name in header index\n", (unsigned)c, (unsigned)c));
 			ascii().addNote("###IndexEntry bad name(ignored)");
 
 			input->seek(pos + cch, librevenge::RVNG_SEEK_SET);
@@ -898,7 +940,7 @@ bool WPS8Parser::parseHeaderIndex()
 	// fixme: sanity check n_entries
 
 	libwps::DebugStream f;
-	f << "Header: N=" << n_entries << ", " << i0 << ", " << i1 << "(";
+	f << "Entries(HeaderZ): N=" << n_entries << ", " << i0 << ", " << i1 << "(";
 
 	for (int i = 0; i < 4; i++)
 		f << std::hex << libwps::read16(input) << std::dec << ",";
@@ -919,7 +961,7 @@ bool WPS8Parser::parseHeaderIndex()
 		uint16_t unknown1 = libwps::readU16(input); // function of the size of the entries ?
 
 		uint16_t n_entries_local = libwps::readU16(input);
-		f << "Header("<<std::hex << unknown1<<"): N=" << std::dec << n_entries_local;
+		f << "HeaderZ("<<std::hex << unknown1<<"): N=" << std::dec << n_entries_local;
 
 		if (n_entries_local > 0x20)
 		{
@@ -985,7 +1027,7 @@ bool WPS8Parser::readDocProperties(WPSEntry const &entry, WPSPageSpan &page)
 
 	if (length < 2)
 	{
-		WPS_DEBUG_MSG(("WPS8Parser::readDocProperties: warning: DOP length=0x%lx\n", length));
+		WPS_DEBUG_MSG(("WPS8Parser::readDocProperties: warning: DOP length=0x%lx\n", (unsigned long) length));
 		return false;
 	}
 
@@ -1246,7 +1288,7 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 
 	if (length < 2)
 	{
-		WPS_DEBUG_MSG(("WPS8Parser::readFRAM warning: length=0x%lx\n", length));
+		WPS_DEBUG_MSG(("WPS8Parser::readFRAM warning: length=0x%lx\n", (unsigned long) length));
 		return false;
 	}
 
@@ -1256,7 +1298,7 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 	int numFram = libwps::read16(input);
 	if (numFram < 0 || numFram*2 > length)
 	{
-		WPS_DEBUG_MSG(("WPS8Parser::readFRAM warning: length=0x%lx, num=%d\n", length, numFram));
+		WPS_DEBUG_MSG(("WPS8Parser::readFRAM warning: length=0x%lx, num=%d\n", (unsigned long) length, numFram));
 		return false;
 	}
 	f << "N=" << numFram;
@@ -1266,7 +1308,7 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 
 
 	bool parsedAll = true, color = false;
-	long lastPos;
+	long lastPos=input->tell(); // uneeded initialization but makes clang analyser happier
 	for (int i = 0; i < numFram; i++)
 	{
 		lastPos = input->tell();
@@ -1356,6 +1398,10 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 			case 1:
 				switch (dt.m_value)
 				{
+				// associated with mailing database with 0x16
+				case 4:
+					frame.m_type = Frame::DbField;
+					break;
 				// associated with 0x13=22:[0|96], 0x18,1b=0x41,0x2e, rarely with 0x1a
 				case 6:
 					frame.m_type = Frame::Header;
@@ -1382,7 +1428,7 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 					break;
 
 				default:
-					f2 << "###type=" << dt.m_value;
+					f2 << "###type=" << dt.m_value << ",";
 					break;
 				}
 				break;
@@ -1440,6 +1486,9 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 				input->seek(actPos, librevenge::RVNG_SEEK_SET);
 				break;
 			}
+			case 0x16: // associated with field and database
+				f2 << "database=" << dt.m_text << ":" << dt.m_value << ",";
+				break;
 			// associated with a tex box and field 0x19
 			case 0x17:
 				color = dt.isTrue();
@@ -1599,6 +1648,189 @@ bool WPS8Parser::readFRAM(WPSEntry const &entry)
 	return false;
 }
 
+/** try to read a frame XXX database  */
+bool WPS8Parser::readFRCD(WPSEntry const &entry)
+{
+	libwps::DebugStream f;
+	if (!entry.hasType(entry.name()))
+	{
+		WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: name=%s, type=%s\n",
+		               entry.name().c_str(), entry.type().c_str()));
+		return false;
+	}
+	RVNGInputStreamPtr input = getInput();
+
+	long page_offset = entry.begin();
+	long length = entry.length();
+	long endPage = entry.end();
+
+	if (length < 4+33+5*8)
+	{
+		WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: length=0x%lx\n", (unsigned long) length));
+		return false;
+	}
+	entry.setParsed();
+	input->seek(page_offset, librevenge::RVNG_SEEK_SET);
+	int val=int(libwps::readU16(input)); // 0x30
+	if (val!=0x30) f << "f0=" << val << ",";
+	int sSz=int(libwps::readU16(input));
+	librevenge::RVNGString str;
+	if (4+2*sSz+33+5*8>length || !m_textParser->readString(input, sSz, str))
+	{
+		WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: can not read main string\n"));
+		return false;
+	}
+	f << str.cstr() << ",";
+	for (int i=0; i<16; ++i)   // f1=10,f2=22,f4=4,f7=256,f11=-1,f12=-256,f13=255,f14=-1
+	{
+		val=int(libwps::read16(input));
+		if (val) f << "f" << i+1 << "=" << val << ",";
+	}
+	val=int(libwps::read8(input));
+	if (val) f << "g0=" << val << ",";
+	ascii().addPos(page_offset);
+	ascii().addNote(f.str().c_str());
+	for (int i=0; i<5; ++i)
+	{
+		long pos=input->tell();
+		f.str("");
+		f << "FRCD-A" << i << ":";
+		if (pos+8>endPage)
+		{
+			WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the zone seems too short\n"));
+			f << "###";
+			ascii().addPos(pos);
+			ascii().addNote(f.str().c_str());
+			return true;
+		}
+		for (int s=0; s<2; ++s)
+		{
+			if (pos+4>endPage || libwps::read8(input)!=-1)
+			{
+				WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the zone seems bad\n"));
+				f << "###";
+				ascii().addPos(pos);
+				ascii().addNote(f.str().c_str());
+				return true;
+			}
+			val=int(libwps::read8(input));
+			if (val==-1)
+			{
+				val=int(libwps::read16(input));
+				if (val!=0x204)
+					f << "f" << s << "=" << val << ",";
+			}
+			else if (val==-2)
+			{
+				val=int(libwps::read8(input));
+				if (val!=-1) f << "g" << s << "=" << val << ",";
+				sSz=int(libwps::readU8(input));
+				str.clear();
+				if (pos+2*sSz>endPage || (sSz && !m_textParser->readString(input, 2*sSz, str)))
+				{
+					WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the zone seems too short\n"));
+					f << "###";
+					ascii().addPos(pos);
+					ascii().addNote(f.str().c_str());
+					return true;
+				}
+				if (!str.empty())
+					f << str.cstr() << ",";
+			}
+			else
+			{
+				WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: find unexpected value\n"));
+				f << "###val=" << val << ",";
+				ascii().addPos(pos);
+				ascii().addNote(f.str().c_str());
+				return true;
+			}
+		}
+		ascii().addPos(pos);
+		ascii().addNote(f.str().c_str());
+	}
+	long pos=input->tell();
+	f.str("");
+	f << "FRCD-B:";
+	if (pos+4>endPage)
+	{
+		WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: can not read the second zone\n"));
+		f << "###";
+		ascii().addPos(pos);
+		ascii().addNote(f.str().c_str());
+		return true;
+	}
+	int N=int(libwps::readU16(input));
+	if (pos+4+10*N>endPage)
+	{
+		WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: can not read the second zone\n"));
+		f << "###";
+		ascii().addPos(pos);
+		ascii().addNote(f.str().c_str());
+		return true;
+	}
+	val=int(libwps::read16(input)); // 0
+	if (val) f << "f0=" << val << ",";
+	ascii().addPos(pos);
+	ascii().addNote(f.str().c_str());
+	for (int n=0; n<N; ++n)
+	{
+		pos=input->tell();
+		f.str("");
+		f << "FRCD-B" << n << ":";
+		if (pos+10>endPage)
+		{
+			WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the zone seems too short\n"));
+			f << "###";
+			ascii().addPos(pos);
+			ascii().addNote(f.str().c_str());
+			return true;
+		}
+		val=int(libwps::read16(input)); // 0
+		if (val) f << "f0=" << val << ",";
+		for (int s=0; s<2; ++s)
+		{
+			if (pos+4>endPage || libwps::read8(input)!=-1)
+			{
+				WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the second zone seems bad\n"));
+				f << "###";
+				ascii().addPos(pos);
+				ascii().addNote(f.str().c_str());
+				return true;
+			}
+			val=int(libwps::read8(input));
+			if (val==-2)
+			{
+				val=int(libwps::read8(input));
+				if (val!=-1) f << "g" << s << "=" << val << ",";
+				sSz=int(libwps::readU8(input));
+				str.clear();
+				if (pos+2*sSz>endPage || (sSz && !m_textParser->readString(input, 2*sSz, str)))
+				{
+					WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: the second zone seems too short\n"));
+					f << "###";
+					ascii().addPos(pos);
+					ascii().addNote(f.str().c_str());
+					return true;
+				}
+				if (!str.empty())
+					f << str.cstr() << ",";
+			}
+			else
+			{
+				WPS_DEBUG_MSG(("WPS8Parser::readFRCD warning: find unexpected value\n"));
+				f << "###val=" << val << ",";
+				ascii().addPos(pos);
+				ascii().addNote(f.str().c_str());
+				return true;
+			}
+		}
+		ascii().addPos(pos);
+		ascii().addNote(f.str().c_str());
+	}
+	return true;
+}
+
 // reads SYID zone: an unknown zone which seems to have the
 //    same number of entries than the text zone
 // also related to the MCLD ?
@@ -1619,7 +1851,7 @@ bool WPS8Parser::readSYID(WPSEntry const &entry, std::vector<int> &listId)
 
 	if (length < 4)
 	{
-		WPS_DEBUG_MSG(("WPS8Parser::readSYID: warning: SYID length=0x%lx\n", length));
+		WPS_DEBUG_MSG(("WPS8Parser::readSYID: warning: SYID length=0x%lx\n", (unsigned long) length));
 		return false;
 	}
 
@@ -1627,7 +1859,7 @@ bool WPS8Parser::readSYID(WPSEntry const &entry, std::vector<int> &listId)
 
 	int unk = libwps::read32(input);
 	int numID = libwps::read32(input);
-	if (numID < 0 || 4*(numID+2) != length)
+	if (numID < 0 || 4*(long(numID)+2) != length)
 	{
 		WPS_DEBUG_MSG(("WPS8Parser::readSYID: invalid length=%ld, num=%d\n", length, numID));
 		return false;
@@ -1666,7 +1898,7 @@ bool WPS8Parser::readWNPR(WPSEntry const &entry)
 
 	if (length < 40)
 	{
-		WPS_DEBUG_MSG(("WPS8Parser::readWNPR: warning: WNPR length=0x%lx\n", length));
+		WPS_DEBUG_MSG(("WPS8Parser::readWNPR: warning: WNPR length=0x%lx\n", (unsigned long) length));
 		return false;
 	}
 
@@ -1716,7 +1948,7 @@ bool WPS8Parser::readWNPR(WPSEntry const &entry)
 	long actPos = input->tell();
 	if (actPos + 32*2+38+22 > endPage)
 	{
-		WPS_DEBUG_MSG(("WPS8Parser::readWNPR: length=0x%lx seems too short\n", length));
+		WPS_DEBUG_MSG(("WPS8Parser::readWNPR: length=0x%lx seems too short\n", (unsigned long) length));
 		return false;
 	}
 
