@@ -22,6 +22,7 @@
 #ifndef WPS8_H
 #define WPS8_H
 
+#include <deque>
 #include <vector>
 
 #include <librevenge/librevenge.h>
@@ -138,6 +139,8 @@ protected:
 	/** \brief reads the FRAM zone: ie a zone which can contains textbox, picture, ...
 		and have some borders */
 	bool readFRAM(WPSEntry const &entry);
+	/** \brief reads the FRCD zone: ie a zone which can contains mailmerge data ? */
+	bool readFRCD(WPSEntry const &entry);
 
 	/** \brief parses a SYID zone
 	 *
@@ -162,6 +165,9 @@ protected:
 	shared_ptr<WPS8Text> m_textParser;
 	//! the internal state
 	shared_ptr<WPS8ParserInternal::State> m_state;
+
+private:
+	std::deque<int> m_sendingTables;
 };
 
 #endif /* WPS8_H */
