@@ -82,6 +82,7 @@ bool WPSCellFormat::convertDTFormat(std::string const &dtFormat, librevenge::RVN
 			break;
 		case 'e':
 			list.insert("number:style", "long");
+		// fall-through intended
 		case 'd':
 			list.insert("librevenge:value-type", "day");
 			propVect.append(list);
@@ -95,6 +96,7 @@ bool WPSCellFormat::convertDTFormat(std::string const &dtFormat, librevenge::RVN
 
 		case 'H':
 			list.insert("number:style", "long");
+		// fall-through intended
 		case 'I':
 			list.insert("librevenge:value-type", "hours");
 			propVect.append(list);
@@ -189,36 +191,33 @@ void WPSCellFormat::addTo(librevenge::RVNGPropertyList &propList) const
 	default:
 		WPS_DEBUG_MSG(("MWAWCell::addTo: called with unknown valign=%d\n", vAlignement()));
 	}
+	if (m_rotation)
+		propList.insert("style:rotation-angle",m_rotation);
 
 	for (size_t c = 0; c < m_bordersList.size(); c++)
 	{
-		std::string property = m_bordersList[c].getPropertyValue();
-		if (property.length() == 0) continue;
+		if (m_bordersList[c].isEmpty()) continue;
 		switch (c)
 		{
 		case WPSBorder::Left:
-			propList.insert("fo:border-left", property.c_str());
+			m_bordersList[c].addTo(propList, "left");
 			break;
 		case WPSBorder::Right:
-			propList.insert("fo:border-right", property.c_str());
+			m_bordersList[c].addTo(propList, "right");
 			break;
 		case WPSBorder::Top:
-			propList.insert("fo:border-top", property.c_str());
+			m_bordersList[c].addTo(propList, "top");
 			break;
 		case WPSBorder::Bottom:
-			propList.insert("fo:border-bottom", property.c_str());
+			m_bordersList[c].addTo(propList, "bottom");
 			break;
 		default:
 			WPS_DEBUG_MSG(("WPSContentListener::openTableCell: can not send %d border\n",int(c)));
 			break;
 		}
 	}
-	if (backgroundColor() != 0xFFFFFF)
-	{
-		char color[20];
-		sprintf(color,"#%06x",backgroundColor());
-		propList.insert("fo:background-color", color);
-	}
+	if (!backgroundColor().isWhite())
+		propList.insert("fo:background-color", backgroundColor().str().c_str());
 	if (m_protected)
 		propList.insert("style:cell-protect","protected");
 }
@@ -273,9 +272,11 @@ bool WPSCellFormat::getNumberingProperties(librevenge::RVNGPropertyList &propLis
 		{
 		case 5: // thousand
 			propList.insert("number:grouping", true);
+		// fall-through intended
 		case 0: // default
 			if (m_subFormat==0)
 				propList.remove("number:decimal-places");
+		// fall-through intended
 		case 1: // decimal
 			propList.insert("librevenge:value-type", "number");
 			break;
@@ -356,8 +357,10 @@ int WPSCellFormat::compare(WPSCellFormat const &cell, bool onlyNumbering) const
 	if (diff) return diff;
 	diff = int(m_vAlign) - int(cell.m_vAlign);
 	if (diff) return diff;
-	diff = int(m_backgroundColor) - int(cell.m_backgroundColor);
-	if (diff) return diff;
+	if (m_rotation<cell.m_rotation) return 1;
+	if (m_rotation>cell.m_rotation) return -1;
+	if (m_backgroundColor<cell.m_backgroundColor) return 1;
+	if (m_backgroundColor>cell.m_backgroundColor) return -1;
 	if (m_protected !=cell.m_protected) return m_protected ? 1 : -1;
 	diff = int(m_bordersList.size()) - int(cell.m_bordersList.size());
 	if (diff) return diff;
@@ -405,6 +408,8 @@ std::ostream &operator<<(std::ostream &o, WPSCellFormat const &cell)
 	default:
 		break; // default
 	}
+	if (cell.m_rotation)
+		o << "rotation=" << cell.m_rotation << ",";
 	int subForm = cell.m_subFormat;
 	switch (cell.m_format)
 	{
@@ -466,12 +471,8 @@ std::ostream &operator<<(std::ostream &o, WPSCellFormat const &cell)
 
 	if (cell.m_digits>-1000) o << "digits=" << cell.m_digits << ",";
 	if (cell.m_protected) o << "protected,";
-	if (cell.m_backgroundColor != 0xFFFFFF)
-	{
-		std::ios::fmtflags oldflags = o.setf(std::ios::hex, std::ios::basefield);
+	if (!cell.m_backgroundColor.isWhite())
 		o << "backColor=" << cell.m_backgroundColor << ",";
-		o.setf(oldflags, std::ios::basefield);
-	}
 	for (size_t i = 0; i < cell.m_bordersList.size(); i++)
 	{
 		if (cell.m_bordersList[i].m_style == WPSBorder::None)
@@ -506,9 +507,9 @@ std::ostream &operator<<(std::ostream &o, WPSCell const &cell)
 	o << "C" << cell.m_position << ":";
 	if (cell.numSpannedCells()[0] != 1 || cell.numSpannedCells()[1] != 1)
 		o << "span=[" << cell.numSpannedCells()[0] << "," << cell.numSpannedCells()[1] << "],";
-	o << "box=" << cell.m_box;
-	if (cell.m_verticalSet) o << "[ySet]";
-	o << ",";
+	if (cell.m_box!=WPSBox2f())
+		o << "box=" << cell.m_box << ",";
+	if (cell.m_verticalSet) o << "ySet,";
 	o << static_cast<WPSCellFormat const &>(cell);
 
 	return o;
