@@ -21,6 +21,7 @@
  */
 
 #include <stdlib.h>
+#include <time.h>
 
 #include <cmath>
 #include <cstdarg>
@@ -181,11 +182,11 @@ bool readDouble8(RVNGInputStreamPtr &input, double &res, bool &isNaN)
 		return false;
 	}
 	input->seek(pos, librevenge::RVNG_SEEK_SET);
-	float mantisse = 0;
+	double mantisse = 0;
 	for (int i = 0; i < 6; i++)
-		mantisse = mantisse/256.f + (float)readU8(input);
+		mantisse = mantisse/256 + (double)readU8(input);
 	int mantExp = (int) readU8(input);
-	mantisse = (mantisse/256.f + float(0x10+(mantExp & 0x0F)))/16.f;
+	mantisse = (mantisse/256 + double(0x10+(mantExp & 0x0F)))/16;
 	int exp = ((mantExp&0xF0)>>4)+int(readU8(input)<<4);
 	int sign = 1;
 	if (exp & 0x800)
@@ -194,16 +195,16 @@ bool readDouble8(RVNGInputStreamPtr &input, double &res, bool &isNaN)
 		sign = -1;
 	}
 
-	float const epsilon=1.e-5f;
+	double const epsilon=1.e-5;
 	if (exp == 0)
 	{
-		if (mantisse > 1.f-epsilon && mantisse < 1.f+epsilon)  return true; // ok zero
+		if (mantisse > 1-epsilon && mantisse < 1+epsilon)  return true; // ok zero
 		// fixme find Nan representation
 		return false;
 	}
 	if (exp == 0x7FF)
 	{
-		if (mantisse >= 1.f-epsilon)
+		if (mantisse >= 1-epsilon)
 		{
 			res=std::numeric_limits<double>::quiet_NaN();
 			return true; // ok 0x7FF and 0xFFF are nan
@@ -232,9 +233,9 @@ bool readDouble10(RVNGInputStreamPtr &input, double &res, bool &isNaN)
 		return false;
 	}
 	input->seek(pos, librevenge::RVNG_SEEK_SET);
-	float mantisse = 0;
+	double mantisse = 0;
 	for (int i = 0; i < 8; i++)
-		mantisse = mantisse/256.f + (float)readU8(input)/128;
+		mantisse = mantisse/256 + (double)readU8(input)/128;
 	int exp = (int)readU16(input);
 	int sign = 1;
 	if (exp & 0x8000)
@@ -243,7 +244,7 @@ bool readDouble10(RVNGInputStreamPtr &input, double &res, bool &isNaN)
 		sign = -1;
 	}
 
-	float const epsilon=1.e-5f;
+	double const epsilon=1.e-5;
 	if (exp == 0)
 	{
 		if (mantisse < epsilon)  return true; // checkme is this zero or a nan
@@ -252,7 +253,7 @@ bool readDouble10(RVNGInputStreamPtr &input, double &res, bool &isNaN)
 	}
 	if (exp == 0x7FFf)
 	{
-		if (mantisse >= 1.f-epsilon)
+		if (mantisse >= 1-epsilon)
 		{
 			res=std::numeric_limits<double>::quiet_NaN();
 			return true; // ok 0x7FF and 0xFFF are nan
@@ -394,7 +395,7 @@ WPSColor WPSColor::barycenter(float alpha, WPSColor const &colA,
 		unsigned char comp= (unsigned char)val;
 		res+=uint32_t(comp<<depl);
 	}
-	return res;
+	return WPSColor(res);
 }
 
 std::ostream &operator<< (std::ostream &o, WPSColor const &c)
@@ -416,6 +417,93 @@ std::string WPSColor::str() const
 	return stream.str();
 }
 
+////////////////////////////////////////////////////////////
+// field
+////////////////////////////////////////////////////////////
+bool WPSField::addTo(librevenge::RVNGPropertyList &propList) const
+{
+	switch (m_type)
+	{
+		break;
+	case PageCount:
+		propList.insert("librevenge:field-type", "text:page-count");
+		propList.insert("style:num-format", numberingTypeToString(m_numberingType).c_str());
+		break;
+	case PageNumber:
+	case PageNumberNext:
+		propList.insert("librevenge:field-type", "text:page-number");
+		propList.insert("style:num-format", numberingTypeToString(m_numberingType).c_str());
+		if (m_type==PageNumberNext)
+			propList.insert("text:select-page", "next");
+		break;
+	case Title:
+		propList.insert("librevenge:field-type", "text:title");
+		break;
+	case Database:
+	case Date:
+	case Link:
+	case Time:
+	case None:
+	default:
+		return false;
+	}
+	return true;
+}
+
+librevenge::RVNGString WPSField::getString() const
+{
+	librevenge::RVNGString res;
+	switch (m_type)
+	{
+	case Database:
+		if (m_data.length())
+			res=librevenge::RVNGString(m_data.c_str());
+		else
+			res=librevenge::RVNGString("#DATAFIELD#");
+		break;
+	case Link:
+		if (m_data.length())
+			res=librevenge::RVNGString(m_data.c_str());
+		else
+			res=librevenge::RVNGString("#LINK#");
+		break;
+	case Title:
+		if (m_data.length())
+			res=librevenge::RVNGString(m_data.c_str());
+		else
+			res=librevenge::RVNGString("#TITLE#");
+		break;
+	case Date:
+	case Time:
+	{
+		std::string format(m_DTFormat);
+		if (format.length()==0)
+		{
+			if (m_type==Date)
+				format="%m/%d/%y";
+			else
+				format="%I:%M:%S %p";
+		}
+		time_t now = time(0L);
+		struct tm timeinfo;
+		if (localtime_r(&now, &timeinfo))
+		{
+			char buf[256];
+			strftime(buf, 256, format.c_str(), &timeinfo);
+			res=librevenge::RVNGString(buf);
+		}
+		break;
+	}
+	case PageCount:
+	case PageNumber:
+	case PageNumberNext:
+	case None:
+	default:
+		break;
+	}
+
+	return res;
+}
 ////////////////////////////////////////////////////////////
 // border
 ////////////////////////////////////////////////////////////
@@ -574,6 +662,169 @@ std::ostream &operator<< (std::ostream &o, WPSBorder const &border)
 	return o;
 }
 
+////////////////////////////////////////////////////////////
+// object
+////////////////////////////////////////////////////////////
+bool WPSEmbeddedObject::addTo(librevenge::RVNGPropertyList &propList) const
+{
+	bool firstSet=false;
+	librevenge::RVNGPropertyListVector auxiliarVector;
+	for (size_t i=0; i<m_dataList.size(); ++i)
+	{
+		if (m_dataList[i].empty()) continue;
+		std::string type=m_typeList.size() ? m_typeList[i] : "image/pict";
+		if (!firstSet)
+		{
+			propList.insert("librevenge:mime-type", type.c_str());
+			propList.insert("office:binary-data", m_dataList[i]);
+			firstSet=true;
+			continue;
+		}
+		librevenge::RVNGPropertyList auxiList;
+		auxiList.insert("librevenge:mime-type", type.c_str());
+		auxiList.insert("office:binary-data", m_dataList[i]);
+		auxiliarVector.append(auxiList);
+	}
+	if (!auxiliarVector.empty())
+		propList.insert("librevenge:replacement-objects", auxiliarVector);
+	if (!firstSet)
+	{
+		WPS_DEBUG_MSG(("WPSEmbeddedObject::addTo: called without picture\n"));
+		return false;
+	}
+	return true;
+}
+
+std::ostream &operator<<(std::ostream &o, WPSEmbeddedObject const &pict)
+{
+	if (pict.isEmpty()) return o;
+	o << "[";
+	for (size_t i=0; i<pict.m_typeList.size(); ++i)
+	{
+		if (pict.m_typeList[i].empty())
+			o << "_,";
+		else
+			o << pict.m_typeList[i] << ",";
+	}
+	o << "],";
+	return o;
+}
+
+////////////////////////////////////////////////////////////
+// unicode
+////////////////////////////////////////////////////////////
+namespace libwps
+{
+void appendUnicode(uint32_t val, librevenge::RVNGString &buffer)
+{
+	if (val < 0x20)
+	{
+		WPS_DEBUG_MSG(("libwps::appendUnicode: find an old char %x, skip it\n", val));
+		return;
+	}
+	uint8_t first;
+	int len;
+	if (val < 0x80)
+	{
+		first = 0;
+		len = 1;
+	}
+	else if (val < 0x800)
+	{
+		first = 0xc0;
+		len = 2;
+	}
+	else if (val < 0x10000)
+	{
+		first = 0xe0;
+		len = 3;
+	}
+	else if (val < 0x200000)
+	{
+		first = 0xf0;
+		len = 4;
+	}
+	else if (val < 0x4000000)
+	{
+		first = 0xf8;
+		len = 5;
+	}
+	else
+	{
+		first = 0xfc;
+		len = 6;
+	}
+
+	char outbuf[7];
+	int i;
+	for (i = len - 1; i > 0; --i)
+	{
+		outbuf[i] = char((val & 0x3f) | 0x80);
+		val >>= 6;
+	}
+	outbuf[0] = char(val | first);
+	outbuf[len] = 0;
+	buffer.append(outbuf);
+}
+}
+
+// a little geometry
+WPSTransformation WPSTransformation::rotation(float angle, Vec2f const &center)
+{
+	float angl=float(double(angle)*M_PI/180);
+	float cosA=float(std::cos(angl));
+	float sinA=float(std::sin(angl));
+	return WPSTransformation(WPSVec3f(cosA, -sinA, center[0]-cosA*center[0]+sinA*center[1]),
+	                         WPSVec3f(sinA, cosA, center[1]-sinA*center[0]-cosA*center[1]));
+}
+
+bool WPSTransformation::decompose(float &rot, Vec2f &shearing, WPSTransformation &transform, Vec2f const &origCenter) const
+{
+	if (m_isIdentity) return false;
+	WPSVec3f const &xRow=(*this)[0];
+	WPSVec3f const &yRow=(*this)[1];
+	Vec2f const &center=*this * origCenter;
+	// first check shearing
+	float shearY=0;
+	float val1=xRow[0]*xRow[1];
+	float val2=yRow[0]*yRow[1];
+	float diff=val2-val1;
+	if (diff<-0.01f || diff>0.01f)
+	{
+		float const &A=val1;
+		float const B=xRow[1]*yRow[0]+xRow[0]*yRow[1];
+		float const &C=diff;
+		if (A>=0 && A<=0)
+		{
+			if (B>=0 && A<=0)
+			{
+				WPS_DEBUG_MSG(("WPSTransformation::decompose: can not determine the shearing\n"));
+				return false;
+			}
+			shearY=C/B;
+		}
+		else
+		{
+			float const &delta=B*B-4*A*C;
+			if (delta<0)
+			{
+				WPS_DEBUG_MSG(("WPSTransformation::decompose: can not determine the shearing\n"));
+				return false;
+			}
+			shearY=(B-float(std::sqrt(delta)))/2.f/A;
+		}
+		transform=WPSTransformation::shear(Vec2f(0,-shearY), center) **this;
+	}
+	else
+		transform=*this;
+	shearing=Vec2f(0,shearY);
+	// fixme: we must first check for symetry here...
+	// now the rotation
+	rot=-std::atan2(-transform[1][0],transform[1][1]);
+	rot *= float(180/M_PI);
+	transform=WPSTransformation::rotation(-rot, center) * transform;
+	return true;
+}
 ////////////////////////////////////////////////////////////
 // debug
 ////////////////////////////////////////////////////////////
